from chromato import constants, parse, utils
from chromato.spaces import Color, RGB


def blend(color1, color2, factor: float = 0.5) -> Color:
    r1, g1, b1 = parse.parse_rgb(color1)
    r2, g2, b2 = parse.parse_rgb(color2)
    r_blend = utils.lerp(r1, r2, factor)
    g_blend = utils.lerp(g1, g2, factor)
    b_blend = utils.lerp(b1, b2, factor)
    return Color(RGB(r_blend, g_blend, b_blend))


def shade(color, factor: float) -> Color:
    black = RGB(0, 0, 0)
    return blend(color, black, factor)


def tone(color, factor: float) -> Color:
    gray = RGB(128, 128, 128)
    return blend(color, gray, factor)


def tint(color, factor: float) -> Color:
    white = RGB(255, 255, 255)
    return blend(color, white, factor)


def grayscale(color) -> Color:
    r, g, b = parse.parse_rgb(color)
    weighted_average = 0.299 * r + 0.587 * g + 0.114 * b
    return Color(RGB(weighted_average, weighted_average, weighted_average))


def invert(color) -> Color:
    rgb = parse.parse_rgb(color)
    r_inv, g_inv, b_inv = [constants.RGB_MAX - v for v in rgb]
    return Color(RGB(r_inv, g_inv, b_inv))


def complement(color) -> Color:
    rgb = parse.parse_rgb(color)
    k = sum([max(*rgb), min(*rgb)])
    r_comp, g_comp, b_comp = [k - v for v in rgb]
    return Color(RGB(r_comp, g_comp, b_comp))


def add(color1, color2) -> Color:
    r1, g1, b1 = parse.parse_rgb(color1)
    r2, g2, b2 = parse.parse_rgb(color2)
    r_sum = min(constants.RGB_MAX, r1 + r2)
    g_sum = min(constants.RGB_MAX, g1 + g2)
    b_sum = min(constants.RGB_MAX, b1 + b2)
    return Color(RGB(r_sum, g_sum, b_sum))


def subtract(color1, color2) -> Color:
    r1, g1, b1 = parse.parse_rgb(color1)
    r2, g2, b2 = parse.parse_rgb(color2)
    r_diff = max(constants.RGB_MIN, r1 - r2)
    g_diff = max(constants.RGB_MIN, g1 - g2)
    b_diff = max(constants.RGB_MIN, b1 - b2)
    return Color(RGB(r_diff, g_diff, b_diff))


def multiply(color1, color2) -> Color:
    r1, g1, b1 = parse.parse_rgb(color1)
    r2, g2, b2 = parse.parse_rgb(color2)
    r_mult = r1 * r2 / constants.RGB_MAX
    g_mult = g1 * g2 / constants.RGB_MAX
    b_mult = b1 * b2 / constants.RGB_MAX
    return Color(RGB(r_mult, g_mult, b_mult))



from collections import UserString

import attr

from chromato import convert, parse


class ColorSpace(object):
    def __iter__(self):
        for val in self.__dict__.values():
            yield val


@attr.s(init=False)
class CMYK(ColorSpace):
    c = attr.ib()
    m = attr.ib()
    y = attr.ib()
    k = attr.ib()

    def __init__(self, *args):
        c, m, y, k = parse.parse_cmyk(*args)
        self.__attrs_init__(c, m, y, k)


class HEX(ColorSpace, UserString):
    def __init__(self, seq: object) -> None:
        super().__init__(parse.parse_hex(seq))


@attr.s(init=False)
class HSL(ColorSpace):
    h = attr.ib()
    s = attr.ib()
    l = attr.ib()

    def __init__(self, *args):
        h, s, l = parse.parse_hsl(*args)
        self.__attrs_init__(h, s, l)


@attr.s(init=False)
class HSV(ColorSpace):
    h = attr.ib()
    s = attr.ib()
    v = attr.ib()

    def __init__(self, *args):
        h, s, v = parse.parse_hsv(*args)
        self.__attrs_init__(h, s, v)


@attr.s(init=False)
class RGB(ColorSpace):
    r = attr.ib()
    g = attr.ib()
    b = attr.ib()

    def __init__(self, *args):
        r, g, b = parse.parse_rgb(*args)
        self.__attrs_init__(r, g, b)


class Color:
    __rgb: RGB

    def __init__(self, *args):
        self.__rgb = RGB(parse.parse_rgb(*args))

    def __eq__(self, other):
        if isinstance(other, Color):
            return self.rgb == other.rgb
        else:
            return False

    def __repr__(self):
        return f"Color(r={self.rgb.r}, g={self.rgb.g}, b={self.rgb.b})"

    @property
    def rgb(self) -> RGB:
        return RGB(*self.__rgb)

    @property
    def hex(self) -> HEX:
        return HEX(convert.rgb_to_hex(self.rgb))

    @property
    def cmyk(self) -> CMYK:
        return CMYK(*convert.rgb_to_cmyk(self.rgb))

    @property
    def hsl(self) -> HSL:
        return HSL(*convert.rgb_to_hsl(self.rgb))

    @property
    def hsv(self) -> HSV:
        return HSV(*convert.rgb_to_hsv(self.rgb))



import numbers
import string

from chromato import constants


def is_bool(value) -> bool:
    return isinstance(value, bool)


def is_int(value) -> bool:
    return not is_bool(value) and isinstance(value, int)


def is_int_in_range(value, range_from, range_to) -> bool:
    return is_int(value) and range_from <= value <= range_to


def is_number(value) -> bool:
    return not isinstance(value, bool) and isinstance(value, numbers.Number)


def is_number_in_range(value, range_from, range_to):
    return is_number(value) and range_from <= value <= range_to


def is_cmyk_value(value) -> bool:
    return is_number_in_range(
        value,
        constants.CMYK_MIN,
        constants.CMYK_MAX,
    )


def is_cmyk(c, m, y, k) -> bool:
    return all(is_cmyk_value(v) for v in (c, m, y, k))


def is_hex(value) -> bool:
    return (
        isinstance(value, str)
        and len(value) == 6
        and all(c in string.hexdigits for c in value)
    )


def is_hsl_value(value) -> bool:
    return is_number_in_range(
        value,
        constants.HSL_MIN,
        constants.HSL_MAX,
    )


def is_hsl(h, s, l) -> bool:
    return all(is_hsl_value(v) for v in (h, s, l))


def is_hsv_value(value) -> bool:
    return is_number_in_range(
        value,
        constants.HSV_MIN,
        constants.HSV_MAX,
    )


def is_hsv(h, s, v) -> bool:
    return all(is_hsv_value(v) for v in (h, s, v))


def is_rgb_value(value) -> bool:
    return is_int_in_range(value, constants.RGB_MIN, constants.RGB_MAX)


def is_rgb(r, g, b) -> bool:
    return all(is_rgb_value(v) for v in (r, g, b))



import math
from chromato import convert, validation, utils
from chromato.spaces import Color, CMYK, HEX, HSL, HSV, RGB


def _is_possibly_hex(arg):
    return isinstance(arg, str) and len(arg.strip("# ")) > 0


def _parse_rgb_values(*args) -> tuple:
    return tuple(map(math.ceil, _parse_float_values(*args)))


def _parse_float_values(*args) -> tuple:
    return tuple([0 if not v else float(v) for v in args])


def parse_hex(_hex) -> str:
    if not _hex:
        return "000000"

    elif isinstance(_hex, HEX):
        return str(_hex)

    elif isinstance(_hex, CMYK):
        return str(convert.cmyk_to_hex(_hex))

    elif isinstance(_hex, HSL):
        return str(convert.hsl_to_hex(_hex))

    elif isinstance(_hex, HSV):
        return str(convert.hsv_to_hex(_hex))

    elif isinstance(_hex, RGB):
        return str(convert.rgb_to_hex(_hex))

    elif isinstance(_hex, Color):
        return str(_hex.hex)

    elif isinstance(_hex, dict) and "hex" in _hex:
        return parse_hex(_hex["hex"])

    try:
        parsed_hex = str(_hex).strip("# ")

        if 1 == len(parsed_hex):
            parsed_hex = 6 * parsed_hex

        elif 3 == len(parsed_hex):
            parsed_hex = "".join(char * 2 for char in parsed_hex)

        if not validation.is_hex(parsed_hex):
            raise

    except BaseException:
        raise ValueError("Unable to parse value as HEX", _hex)

    return parsed_hex


def parse_hsv(*args) -> tuple:
    h, s, v = (0, 0, 0)

    try:
        num_args = len(args)

        if 1 == num_args:
            arg = args[0]

            if isinstance(arg, HSV):
                return tuple(arg)

            elif isinstance(arg, HEX):
                return tuple(convert.hex_to_hsv(arg))

            elif isinstance(arg, CMYK):
                return tuple(convert.cmyk_to_hsv(arg))

            elif isinstance(arg, HSL):
                return tuple(convert.hsl_to_hsv(arg))

            elif isinstance(arg, RGB):
                return tuple(convert.rgb_to_hsv(arg))

            elif isinstance(arg, Color):
                return tuple(arg.hsv)

            elif type(arg) in [tuple, list]:
                return parse_hsv(*arg)

            elif isinstance(arg, dict) and utils.dict_has_keys(arg, "hsv"):
                h, s, v = arg["h"], arg["s"], arg["v"]

            elif _is_possibly_hex(arg):
                h, s, v = convert.hex_to_hsv(parse_hex(arg))

            else:
                h = arg

        elif 2 == num_args:
            h, s = args

        elif 3 == num_args:
            h, s, v = args

        h, s, v = _parse_float_values(h, s, v)

        if not validation.is_hsv(h, s, v):
            raise

    except BaseException:
        raise ValueError("Unable to parse value as HSV", args)

    return h, s, v


def parse_hsl(*args) -> tuple:
    h, s, l = (0, 0, 0)

    try:
        num_args = len(args)

        if 1 == num_args:
            arg = args[0]

            if isinstance(arg, HSL):
                return tuple(arg)

            elif isinstance(arg, HEX):
                return tuple(convert.hex_to_hsl(arg))

            elif isinstance(arg, CMYK):
                return tuple(convert.cmyk_to_hsl(arg))

            elif isinstance(arg, HSV):
                return tuple(convert.hsv_to_hsl(arg))

            elif isinstance(arg, RGB):
                return tuple(convert.rgb_to_hsl(arg))

            elif isinstance(arg, Color):
                return tuple(arg.hsl)

            elif type(arg) in [tuple, list]:
                return parse_hsl(*arg)

            elif isinstance(arg, dict) and utils.dict_has_keys(arg, "hsl"):
                h, s, l = arg["h"], arg["s"], arg["l"]

            elif _is_possibly_hex(arg):
                h, s, l = convert.hex_to_hsl(parse_hex(arg))

            else:
                h = arg

        elif 2 == num_args:
            h, s = args

        elif 3 == num_args:
            h, s, l = args

        h, s, l = _parse_float_values(h, s, l)

        if not validation.is_hsl(h, s, l):
            raise

    except BaseException:
        raise ValueError("Unable to parse value as HSL", args)

    return h, s, l


def parse_cmyk(*args) -> tuple:
    c, m, y, k = (0, 0, 0, 100)

    try:
        num_args = len(args)

        if 1 == num_args:
            arg = args[0]

            if isinstance(arg, CMYK):
                return tuple(arg)

            elif isinstance(arg, HEX):
                return tuple(convert.hex_to_cmyk(arg))

            elif isinstance(arg, HSL):
                return tuple(convert.hsl_to_cmyk(arg))

            elif isinstance(arg, HSV):
                return tuple(convert.hsv_to_cmyk(arg))

            elif isinstance(arg, RGB):
                return tuple(convert.rgb_to_cmyk(arg))

            elif isinstance(arg, Color):
                return tuple(arg.cmyk)

            elif type(arg) in [tuple, list]:
                return parse_cmyk(*arg)

            elif isinstance(arg, dict) and utils.dict_has_keys(arg, "cmyk"):
                c, m, y, k = arg["c"], arg["m"], arg["y"], arg["k"]

            elif _is_possibly_hex(arg):
                c, m, y, k = convert.hex_to_cmyk(parse_hex(arg))

            else:
                c = arg

        elif 2 == num_args:
            c, m = args

        elif 3 == num_args:
            c, m, y = args

        elif 4 == num_args:
            c, m, y, k = args

        c, m, y, k = _parse_float_values(c, m, y, k)

        if not validation.is_cmyk(c, m, y, k):
            raise

    except BaseException:
        raise ValueError("Unable to parse value as CMYK", args)

    return c, m, y, k


def parse_rgb(*args) -> tuple:
    r, g, b = (0, 0, 0)

    try:
        num_args = len(args)

        if 1 == num_args:
            arg = args[0]

            if isinstance(arg, RGB):
                return tuple(arg)

            elif isinstance(arg, CMYK):
                return tuple(convert.cmyk_to_rgb(arg))

            elif isinstance(arg, HSV):
                return tuple(convert.hsv_to_rgb(arg))

            elif isinstance(arg, HSL):
                return tuple(convert.hsl_to_rgb(arg))

            elif isinstance(arg, HEX):
                return tuple(convert.hex_to_rgb(arg))

            elif isinstance(arg, Color):
                return tuple(arg.rgb)

            elif type(arg) in [tuple, list]:
                return parse_rgb(*arg)

            elif isinstance(arg, dict) and utils.dict_has_keys(arg, "rgb"):
                r, g, b = arg["r"], arg["g"], arg["b"]

            elif _is_possibly_hex(arg):
                r, g, b = convert.hex_to_rgb(parse_hex(arg))

            else:
                r = arg

        elif 2 == num_args:
            r, g = args

        elif 3 == num_args:
            r, g, b = args

        r, g, b = _parse_rgb_values(r, g, b)

        if not validation.is_rgb(r, g, b):
            raise

    except BaseException:
        raise ValueError("Unable to parse value as RGB", args)

    return r, g, b



import colorsys

from chromato import constants, parse, spaces


def cmyk_to_hex(*args) -> spaces.HEX:
    rgb = cmyk_to_rgb(*args)
    return rgb_to_hex(rgb)


def cmyk_to_hsl(*args) -> spaces.HSL:
    rgb = cmyk_to_rgb(*args)
    return rgb_to_hsl(rgb)


def cmyk_to_hsv(*args) -> spaces.HSV:
    rgb = cmyk_to_rgb(*args)
    return rgb_to_hsv(rgb)


def cmyk_to_rgb(*args) -> spaces.RGB:
    c, m, y, k = parse.parse_cmyk(*args)
    r = constants.RGB_MAX * (1.0 - (c + k) / constants.CMYK_MAX)
    g = constants.RGB_MAX * (1.0 - (m + k) / constants.CMYK_MAX)
    b = constants.RGB_MAX * (1.0 - (y + k) / constants.CMYK_MAX)
    return spaces.RGB(r, g, b)


def float_to_hex(_float: float) -> str:
    template = "{:02x}" if _float < 16 else "{:x}"
    return template.format(int(_float))


def hex_to_int(_hex: str) -> int:
    return int(_hex, 16)


def hex_to_cmyk(_hex) -> spaces.CMYK:
    rgb = hex_to_rgb(_hex)
    return rgb_to_cmyk(rgb)


def hex_to_hsl(_hex) -> spaces.HSL:
    rgb = hex_to_rgb(_hex)
    return rgb_to_hsl(rgb)


def hex_to_hsv(_hex) -> spaces.HSV:
    rgb = hex_to_rgb(_hex)
    return rgb_to_hsv(rgb)


def hex_to_rgb(_hex) -> spaces.RGB:
    result = parse.parse_hex(_hex)
    hlen = len(result)
    pairs = hlen // 3
    r, g, b = [hex_to_int(result[i : i + pairs]) for i in range(0, hlen, pairs)]
    return spaces.RGB(r, g, b)


def hsl_to_cmyk(*args) -> spaces.CMYK:
    rgb = hsl_to_rgb(*args)
    return rgb_to_cmyk(rgb)


def hsl_to_hex(*args) -> spaces.HEX:
    rgb = hsl_to_rgb(*args)
    return rgb_to_hex(rgb)


def hsl_to_hsv(*args) -> spaces.HSV:
    rgb = hsl_to_rgb(*args)
    return rgb_to_hsv(rgb)


def hsl_to_rgb(*args) -> spaces.RGB:
    h, s, l = parse.parse_hsl(*args)
    r, g, b = [constants.RGB_MAX * v for v in colorsys.hls_to_rgb(h, l, s)]
    return spaces.RGB(r, g, b)


def hsv_to_cmyk(*args) -> spaces.CMYK:
    rgb = hsv_to_rgb(*args)
    return rgb_to_cmyk(rgb)


def hsv_to_hex(*args) -> spaces.HEX:
    rgb = hsv_to_rgb(*args)
    return rgb_to_hex(rgb)


def hsv_to_hsl(*args) -> spaces.HSL:
    rgb = hsv_to_rgb(*args)
    return rgb_to_hsl(rgb)


def hsv_to_rgb(*args) -> spaces.RGB:
    hsv = parse.parse_hsv(*args)
    r, g, b = [constants.RGB_MAX * v for v in colorsys.hsv_to_rgb(*hsv)]
    return spaces.RGB(r, g, b)


def rgb_to_cmyk(*args) -> spaces.CMYK:
    r, g, b = parse.parse_rgb(*args)

    if (r, g, b) == (constants.RGB_MIN, constants.RGB_MIN, constants.RGB_MIN):
        return spaces.CMYK(
            constants.CMYK_MIN,
            constants.CMYK_MIN,
            constants.CMYK_MIN,
            constants.CMYK_MAX,
        )  # black

    # rgb [0,255] -> cmy [0,1]
    c = 1 - (r / constants.RGB_MAX)
    m = 1 - (g / constants.RGB_MAX)
    y = 1 - (b / constants.RGB_MAX)

    # extract out k [0,1]
    k = min(c, m, y)
    c = c - k
    m = m - k
    y = y - k

    # rescale to the range [0,100]
    c, m, y, k = [v * constants.CMYK_MAX for v in (c, m, y, k)]
    return spaces.CMYK(c, m, y, k)


def rgb_to_hex(*args) -> spaces.HEX:
    parsed_rgb = parse.parse_rgb(*args)
    _hex = "".join(map(float_to_hex, parsed_rgb))
    return spaces.HEX(_hex)


def rgb_to_hsl(*args) -> spaces.HSL:
    rgb = parse.parse_rgb(*args)
    h, l, s = colorsys.rgb_to_hls(*[v / constants.RGB_MAX for v in rgb])
    return spaces.HSL(h, s, l)


def rgb_to_hsv(*args) -> spaces.HSV:
    rgb = parse.parse_rgb(*args)
    h, s, v = colorsys.rgb_to_hsv(*[v / constants.RGB_MAX for v in rgb])
    return spaces.HSV(h, s, v)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chromato']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'chromato',
    'version': '0.4.0',
    'description': '🍅 Fresh color utilities.',
    'long_description': '<div align="center">\n    <h1>🍅 chromato</h1>\n    <p>\n        <b>Fresh color utilities for Python</b>\n    </p>\n\n[![PyPI version](https://badge.fury.io/py/chromato.svg)](https://pypi.org/project/chromato/) [![test](https://github.com/vikpe/chromato/actions/workflows/test.yml/badge.svg)](https://github.com/vikpe/chromato/actions/workflows/test.yml) [![codecov](https://codecov.io/gh/vikpe/chromato/branch/main/graph/badge.svg)](https://codecov.io/gh/vikpe/chromato)\n\n<br>\n\n</div>\n\n* [**Color spaces**](#color-spaces): CMYK, HEX, RGB, HSL, HSV\n* [**Color class**](#color-class): Convenience class for color manipulation\n* [**Operations**](#operations): shade, tone, tint, grayscale, invert, complement, add, subtract, multiply\n* [**Conversion**](#conversion): Convert any color space to any color space\n* [Parsing](#parsing) \n* [Validation](#validation)\n* (zero dependecies!)\n\n<br>\n\n# Install\n\n```shell\npip install chromato\n```\n\n<br>\n\n# Example usage\n\n**Color class**\n```python\nfrom chromato.spaces import Color\n\nred = Color(255, 0, 0)\n\nred.cmyk  # CMYK(c=0, m=100, y=100, k=0)\nred.hex   # HEX(ff0000)\nred.rgb   # RGB(r=255, g=0, b=0)\nred.hsl   # HSL(h=0, s=1, l=0.5)\nred.hsv   # HSV(h=0, s=1, v=1)\n```\n\n**Operations**\n```python\nfrom chromato import operations\nfrom chromato.spaces import RGB\n\nred = RGB(255, 0, 0)\nblue = RGB(0, 0, 255)\n\noperations.invert(red).rgb       # RGB(r=0, g=255, b=255)\noperations.add(red, blue)        # RGB(r=255, g=0, b=255)\noperations.blend(red, blue).rgb  # RGB(r=128, g=0, b=128)\noperations.tint(red, 0.1).rgb    # RGB(r=255, g=26, b=26)\n```\n\n**Conversion**\n```python\nfrom chromato import convert\n\nconvert.rgb_to_hex(255, 0, 0)  # HEX(ff0000)\nconvert.hex_to_rgb("ff0000")   # RGB(r=255, g=0, b=0)\nconvert.hex_to_cmyk("f0f")     # CMYK(c=0, m=100, y=0, k=0)\n```\n\n<br>\n\n# API\n\n## Color spaces\n\nName | Properties | Range\n---|---|---\nCMYK | `c`, `m`, `y`, `k` | `0-100`, `0-100`, `0-100`, `0-100`\nHEX | (is a string) | `000000-ffffff`\nHSL | `h`, `s`, `l` |  `0-1`, `0-1`, `0-1`\nHSV | `h`, `s`, `v` |  `0-1`, `0-1`, `0-1`\nRGB | `r`, `g`, `b` | `0-255`, `0-255`, `0-255`\n\n```python\nfrom chromato.spaces import CMYK, HEX, HSL, HSV, RGB\n\nred_cmyk = CMYK(0, 100, 100, 0)\nred_hex  = HEX("ff0000")\nred_hsl  = HSL(0, 1, 0.5)\nred_hsv  = HSV(0, 1, 1)\nred_rgb  = RGB(255, 0, 0)\n```\n\n## Color class\n\n**Properties**\n```python\nfrom chromato.spaces import Color\n\nred = Color(255, 0, 0)\n\nred.cmyk  # CMYK(c=0, m=100, y=100, k=0)\nred.hex   # HEX(ff0000)\nred.hsl   # HSL(h=0, s=1, l=0.5)\nred.hsv   # HSV(h=0, s=1, v=1)\nred.rgb   # RGB(r=255, g=0, b=0)\n```\n\n**Construct**\n\n```python\n# examples below are equal\nColor(255, 0, 0)\nColor((255, 0, 0))\nColor([255, 0, 0])\nColor({"r": 255, "g": 0, "b": 0})\nColor("ff0000")\nColor("ff0")\nColor(Color(255, 0, 0))\nColor(RGB(255, 0, 0))\nColor(HEX("ff0"))\nColor(HSV(0, 1, 1))\nColor(HSL(0, 1, 0.5))\nColor(CMYK(0, 100, 100, 0))\n```\n\n## Operations\nEach operation take one or several color values and returns a `Color` instance.\n\nOperation | Description\n--- | ---\n**`blend`**`(color1, color2, factor)` | Blend/mix colors\n**`tint`**`(color, factor)` | Increase lightness (blend with white)\n**`shade`**`(color, factor)` | Increase darkness (blend with black)\n**`tone`**`(color, factor)` | Reduce colorfullness (blend with gray)\n**`invert`**`(color)` | Invert color\n**`complement`**`(color)` | Complementary color\n**`add`**`(color1, color2)` | Add colors\n**`subtract`**`(color1, color2)` | Subtract colors\n**`multiply`**`(color1, color2)` | Multiply colors\n\n* **Arguments**: `color<any>`, `factor <float> [0-1]`\n* **Returns**: instance of `Color`\n\n**Example**\n```python\nfrom chromato.spaces import RGB\nfrom chromato.operations import blend, invert\n\nwhite = RGB(255, 255, 255)\nblack = RGB(0, 0, 0)\n\nblend(white, black).rgb       # RGB(r=128, g=128, b=128)\nblend(white, black, 0.2).rgb  # RGB(r=204, g=204, b=204)\nblend(white, black, 0.8).rgb  # RGB(r=51, g=51, b=51)\n\ninvert(black).rgb             # RGB(r=255, g=255, b=255)\ninvert((255, 0, 0)).cmyk      # CMYK(c=0, m=100, y=100, k=0)\ninvert("ff0000").hex          # HEX(00ffff)\n```\n\n## Conversion\nConvert any color space to any color space.\n\n🔀 | RGB | HEX | CMYK | HSL | HSV\n--- | --- | --- | --- | --- | ---\nRGB | <!-- null --> | `hex_to_rgb` | `cmyk_to_rgb` | `hsl_to_rgb` | `hsv_to_rgb`\nHEX | `rgb_to_hex` | <!-- null --> | `cmyk_to_hex` | `hsl_to_hex` | `hsv_to_hex`\nCMYK | `rgb_to_cmyk` | `hex_to_cmyk` | <!-- null --> | `hsl_to_cmyk` | `hsv_to_cmyk`\nHSL | `rgb_to_hsl` | `hex_to_hsl` | `cmyk_to_hsl` | <!-- null --> | `hsv_to_hsl`\nHSV | `rgb_to_hsv` | `hex_to_hsv` | `cmyk_to_hsv` | `hsl_to_hsv` | <!-- null -->\n\n\n**Example**\n\n```python\nfrom chromato import convert\n\nconvert.rgb_to_cmyk(255, 0, 0)  # CMYK(c=0, m=100, y=100, k=0)\nconvert.rgb_to_hex(255, 0, 0)   # HEX(ff0000)\nconvert.rgb_to_hex(255, 0, 0)   # HEX(ff0000)\nconvert.rgb_to_hsl(255, 0, 0)   # HSL(h=0, s=1, l=0.5)\nconvert.rgb_to_hsv(255, 0, 0)   # HSV(h=0, s=1, v=1)\n```\n\n## Parsing\nEach parse function takes any kind of value and tries to parse it.\n\nFunction | Returns | Description\n--- | --- | ---\n**`parse_cmyk`**`(value)` | `tuple(c,m,y,k)` |  Parse value as CMYK \n**`parse_hex`**`(value)` | `str(hex)` |  Parse value as HEX\n**`parse_hsl`**`(value)` | `tuple(h,l,s)` |  Parse value as HSL \n**`parse_hsv`**`(value)` | `tuple(h,s,v)` |  Parse value as HSV \n**`parse_rgb`**`(value)` | `tuple(r,g,b)` |  Parse value as RGB \n\n**Example**\n\n```python\nfrom chromato import parse\nfrom chromato.spaces import Color, HEX, RGB\n\nparse.parse_hex("f")                 # "ffffff"\nparse.parse_hex("f60")               # "ff6600"\nparse.parse_hex("ff6600")            # "ff6600"\nparse.parse_hex(" #ff6600 ")         # "ff6600"\nparse.parse_hex(333)                 # "333333"\nparse.parse_hex(HEX("ff6600"))       # "ff6600"\nparse.parse_hex(RGB(255, 102, 0))    # "ff6600"\nparse.parse_hex(Color(255, 102, 0))  # "ff6600"\n```\n\n## Validation\nEach validation function validates type and range. Returns `True`/`False`.\n\n```python\nfrom chromato import validation\n\nvalidation.is_cmyk(c, m, y, k)\nvalidation.is_hex(hex)\nvalidation.is_hsl(h, s, l)\nvalidation.is_hsv(h, s, v)\nvalidation.is_rgb(r, g, b)\n```\n\n<br>\n\n# Development [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n**Setup**\n\n1. Install [poetry](https://github.com/python-poetry/poetry)\n2. `poetry install`\n\n**Tests**\n\nRun tests on changes in source code or tests.\n\n```shell\npoetry run ptw --clear --runner "poetry run pytest --cov -vv" \n```\n\n**Code formatting (black)**\n\n```shell\npoetry run black . \n```\n',
    'author': 'Viktor Persson',
    'author_email': 'viktor.persson@arcsin.se',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/vikpe/chromato',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
